# Generated by roxygen2: do not edit by hand

S3method("$<-",stars)
S3method("[",dimension)
S3method("[",dimensions)
S3method("[",intervals)
S3method("[",intervals_list)
S3method("[",mdim)
S3method("[",nc_proxy)
S3method("[",stars)
S3method("[",stars_proxy)
S3method("[<-",dimensions)
S3method("[<-",stars)
S3method("[<-",stars_proxy)
S3method("[[<-",stars)
S3method("[[<-",stars_proxy)
S3method("dimnames<-",stars)
S3method("st_crs<-",dimensions)
S3method("st_crs<-",stars)
S3method("st_dimensions<-",list)
S3method("st_dimensions<-",stars)
S3method("st_dimensions<-",stars_proxy)
S3method("st_geotransform<-",stars)
S3method(Math,stars)
S3method(Math,stars_proxy)
S3method(Ops,stars)
S3method(Ops,stars_proxy)
S3method(adrop,stars)
S3method(adrop,stars_proxy)
S3method(aggregate,stars)
S3method(aggregate,stars_proxy)
S3method(all.equal,dimensions)
S3method(aperm,stars)
S3method(aperm,stars_proxy)
S3method(as.Date,stars)
S3method(as.POSIXct,stars)
S3method(as.data.frame,dimensions)
S3method(as.data.frame,stars)
S3method(as.data.frame,stars_proxy)
S3method(as.list,intervals)
S3method(asub,factor)
S3method(c,intervals)
S3method(c,nc_proxy)
S3method(c,stars)
S3method(c,stars_proxy)
S3method(contour,stars)
S3method(cut,array)
S3method(cut,matrix)
S3method(cut,stars)
S3method(dim,dimensions)
S3method(dim,stars)
S3method(dim,stars_proxy)
S3method(dimnames,stars)
S3method(drop_units,dimension)
S3method(drop_units,stars)
S3method(droplevels,stars)
S3method(droplevels,stars_proxy)
S3method(expand_dimensions,dimensions)
S3method(expand_dimensions,stars)
S3method(format,intervals)
S3method(format,intervals_list)
S3method(head,intervals)
S3method(hist,stars)
S3method(hist,stars_proxy)
S3method(image,stars)
S3method(image,stars_proxy)
S3method(is.na,stars)
S3method(is.na,stars_proxy)
S3method(length,intervals)
S3method(merge,stars)
S3method(merge,stars_proxy)
S3method(plot,nc_proxy)
S3method(plot,stars)
S3method(plot,stars_proxy)
S3method(prcomp,stars)
S3method(prcomp,stars_proxy)
S3method(predict,stars)
S3method(predict,stars_proxy)
S3method(print,dimensions)
S3method(print,nc_proxy)
S3method(print,stars)
S3method(print,stars_proxy)
S3method(print,stars_raster)
S3method(range,dimension)
S3method(seq,dimension)
S3method(split,stars)
S3method(split,stars_proxy)
S3method(st_apply,stars)
S3method(st_apply,stars_proxy)
S3method(st_area,stars)
S3method(st_as_sf,dimensions)
S3method(st_as_sf,stars)
S3method(st_as_sf,stars_proxy)
S3method(st_as_sfc,stars)
S3method(st_as_stars,CFDataset)
S3method(st_as_stars,CFVariable)
S3method(st_as_stars,OpenStreetMap)
S3method(st_as_stars,Raster)
S3method(st_as_stars,STF)
S3method(st_as_stars,STFDF)
S3method(st_as_stars,STS)
S3method(st_as_stars,STSDF)
S3method(st_as_stars,SpatRaster)
S3method(st_as_stars,Spatial)
S3method(st_as_stars,bbox)
S3method(st_as_stars,cubble_df)
S3method(st_as_stars,data.frame)
S3method(st_as_stars,default)
S3method(st_as_stars,im)
S3method(st_as_stars,list)
S3method(st_as_stars,mdim)
S3method(st_as_stars,nc_proxy)
S3method(st_as_stars,ncdfgeom)
S3method(st_as_stars,sf)
S3method(st_as_stars,sfc)
S3method(st_as_stars,stars)
S3method(st_as_stars,stars_proxy)
S3method(st_as_stars,xts)
S3method(st_bbox,default)
S3method(st_bbox,dimensions)
S3method(st_bbox,stars)
S3method(st_coordinates,dimensions)
S3method(st_coordinates,stars)
S3method(st_crop,mdim)
S3method(st_crop,nc_proxy)
S3method(st_crop,stars)
S3method(st_crop,stars_proxy)
S3method(st_crs,dimensions)
S3method(st_crs,stars)
S3method(st_dimensions,array)
S3method(st_dimensions,default)
S3method(st_dimensions,dimensions)
S3method(st_dimensions,matrix)
S3method(st_dimensions,stars)
S3method(st_downsample,stars)
S3method(st_downsample,stars_proxy)
S3method(st_extract,stars)
S3method(st_geometry,stars)
S3method(st_geotransform,default)
S3method(st_geotransform,dimensions)
S3method(st_geotransform,stars)
S3method(st_interpolate_aw,stars)
S3method(st_intersects,bbox)
S3method(st_intersects,stars)
S3method(st_join,stars)
S3method(st_mosaic,character)
S3method(st_mosaic,nc_proxy)
S3method(st_mosaic,stars)
S3method(st_mosaic,stars_proxy)
S3method(st_normalize,stars)
S3method(st_normalize,stars_proxy)
S3method(st_redimension,nc_proxy)
S3method(st_redimension,stars)
S3method(st_redimension,stars_proxy)
S3method(st_rotate,sf)
S3method(st_rotate,sfc)
S3method(st_rotate,stars)
S3method(st_sample,stars)
S3method(st_sample,stars_proxy)
S3method(st_set_bbox,dimensions)
S3method(st_set_bbox,stars)
S3method(st_set_bbox,stars_proxy)
S3method(st_transform,stars)
S3method(st_write,stars)
S3method(tail,intervals)
S3method(time,stars)
S3method(units,dimension)
S3method(write_stars,stars)
S3method(write_stars,stars_proxy)
export("st_dimensions<-")
export("st_geotransform<-")
export(as.tbl_cube.stars)
export(detect.driver)
export(expand_dimensions)
export(geom_stars)
export(make_intervals)
export(read_mdim)
export(read_ncdf)
export(read_stars)
export(st_apply)
export(st_as_stars)
export(st_cells)
export(st_contour)
export(st_dim_to_attr)
export(st_dimensions)
export(st_downsample)
export(st_extract)
export(st_flip)
export(st_geotransform)
export(st_get_dimension_values)
export(st_mosaic)
export(st_raster_type)
export(st_rasterize)
export(st_redimension)
export(st_res)
export(st_rgb)
export(st_rotate)
export(st_set_bbox)
export(st_set_dimensions)
export(st_sfc2xy)
export(st_tile)
export(st_warp)
export(st_xy2sfc)
export(write_mdim)
export(write_stars)
exportMethods("%in%")
import(sf)
import(units)
importFrom(abind,abind)
importFrom(abind,adrop)
importFrom(abind,asub)
importFrom(grDevices,cm)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.capabilities)
importFrom(grDevices,dev.size)
importFrom(grDevices,grey)
importFrom(grDevices,rgb)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,image.default)
importFrom(graphics,layout)
importFrom(graphics,lcm)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,rasterImage)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,as)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(parallel,parApply)
importFrom(rlang,"%||%")
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,time)
importFrom(stats,var)
importFrom(tools,file_ext)
importFrom(utils,head)
importFrom(utils,methods)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
