% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stars.R, R/tidyverse.R
\name{st_coordinates}
\alias{st_coordinates}
\alias{st_coordinates.stars}
\alias{as.data.frame.stars}
\alias{as_tibble.stars}
\title{retrieve coordinates for raster or vector cube cells}
\usage{
\method{st_coordinates}{stars}(x, ..., add_max = FALSE, center = TRUE)

\method{as.data.frame}{stars}(x, ..., add_max = FALSE, center = NA, add_coordinates = TRUE)

as_tibble.stars(.x, ..., add_max = FALSE, center = NA)
}
\arguments{
\item{x}{object of class \code{stars}}

\item{...}{ignored}

\item{add_max}{logical; if \code{TRUE}, dimensions are given with a min (x) and max (x_max) value}

\item{center}{logical; (only if \code{add_max} is FALSE): should grid cell center coordinates be returned (TRUE) or offset values (FALSE)? \code{center} can be a named logical vector or list to specify values for each dimension.}

\item{add_coordinates}{logical; if `TRUE`, columns with dimension values preceed the array values, 
otherwise they are omitted}

\item{.x}{object to be converted to a tibble}
}
\description{
retrieve coordinates for raster or vector cube cells
}
